from multiprocessing import Pool

import subprocess
import os
import json
import tempfile

from os.path import basename
from ..common.redisbc import RedisBC

from ..common.database import Database


def retrieve_stdout(command):
    proc = subprocess.Popen(command, stdout=subprocess.PIPE)
    output = proc.stdout.read()
    RedisBC().connect("localhost",6379)
    RedisBC().publish("hola",output.decode('utf-8').strip())
    return output.decode('utf-8').strip()

def process_job(meta, oauth_token):
    RedisBC().connect("localhost",6379)
    RedisBC().publish("hola","processant...")
    instance_path = tempfile.mkdtemp()
    instance_name = basename(instance_path)

    retrieve_stdout(['docker', 'create', '-t', '--rm',
        '-e', 'GITHUB_BRANCH=' + meta['branch'],
        '-e', 'GITHUB_ORGANIZATION=' + meta['org']['name'],
        '-e', 'GITHUB_REPOSITORY=' + meta['repo']['name'],
        '-e', 'GITHUB_ORGANIZATION_ID=' + str(meta['org']['id']),
        '-e', 'GITHUB_REPOSITORY_ID=' + str(meta['repo']['id']),
        '-e', 'GITHUB_COMMIT=' + meta['hash'],
        '-e', 'OAUTH_TOKEN=' + oauth_token,
        '-e', 'DOCKER_NAME=' + instance_name,
        '-e', 'AUTOGRADER_SECRET=' + Database().get_organization_config(meta['org']['id'])['secret'],
        '-v', '/var/run/docker.sock:/var/run/docker.sock',
        '--mount', 'type=tmpfs,destination=/instance',
        '--network', 'backend',
        '--name', instance_name,
        'agent-bootstrap'])

    retrieve_stdout(['docker', 'network', 'connect', 'results', instance_name])
    
    return meta, retrieve_stdout(['docker', 'start', '-a', instance_name])
    
class Jobs(object):
    __instance = None

    def __new__(cls):
        if Jobs.__instance is None:
            Jobs.__instance = object.__new__(cls)
        return Jobs.__instance

    def __init__(self):
        RedisBC().connect("localhost",6379)
        RedisBC().publish("hola","iniciant...")
        config = Database().get_config()
        self.oauth_token = config['oauth_token']
        self.pool = Pool(config['parallel_jobs'])
    
    def __once_done(self, result):
        meta, log = result
        RedisBC().connect("localhost",6379)
        RedisBC().publish("hola",log)
        Database().set_instance_log(meta['org']['id'], meta['repo']['id'], meta['hash'], meta['branch'], log)
        return True
    
    def post(self, meta):
        RedisBC().connect("localhost",6379)
        RedisBC().publish("hola","corretgint...")
        if os.environ.get('DISABLE_POOL'):
            return self.__once_done(process_job(meta, self.oauth_token))
        else:
            return self.pool.apply_async(process_job, (meta, self.oauth_token), callback=self.__once_done)
